<!DOCTYPE html>
<html>
<head>
<title>Profile view</title>
<script src="//cdnjs.cloudflare.com/ajax/libs/d3/3.4.10/d3.min.js"></script>
<script>
{% raw handler.embed_file("force.js") %}
</script>
</head>
<body>
  <svg id="graph">
    <defs>
      <!-- from http://www.w3.org/TR/SVG/painting.html#Markers -->
      <!-- Increasing refX moves the marker away from the end of the line -->
      <marker id="Triangle"
              viewBox="0 0 10 10" refX="30" refY="5"
              markerUnits="strokeWidth"
              markerWidth="4" markerHeight="3"
              orient="auto">
        <path d="M 0 0 L 10 5 L 0 10 z" />
      </marker>
    </defs>
  </svg>
  <script>
    var data = {% raw json_encode(data) %};
    dataReady(data);
  </script>
</body>
</html>
