<!DOCTYPE html>
<html>
<head>
<title>Profile Viewer</title>
</head>
<body>
  <ul>
    {% for f in files %}
    <li><a href="/view?filename={{f}}">{{f}}</a></li>
    {% end for %}
  </ul>
</body>
</html>
